@@ -2,7 +2,7 @@
                           cdp.cpp  -  description
                              -------------------
     begin                : Sun Aug 26 2001
-    copyright            : (C) 2001 by Thomas Friedrichsmeier
+    copyright            : (C) 2001, 2002 by Thomas Friedrichsmeier
     email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
  ***************************************************************************/
 
@@ -14,42 +14,48 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
+#include "defines.h"
 
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qstrlist.h>
 #include <qstring.h>
 #include <qdom.h>
+#include <qtextstream.h>
+#include <qregexp.h>
 
 #include <qwidget.h>
 
 #include <klocale.h>
 #include <kconfig.h>
 #include <kglobal.h>
-#include <kstddirs.h>
+#include KSTDDIRS
 #include <kcmdlineargs.h>
 #include <kmessagebox.h>
-#include <kapp.h>
+#include KAPP
 
 #include <stdlib.h>
+#include <unistd.h>
 
 #include "cdp.h"
+#include "taxipilot.h"
+#include "taxipilotapp.h"
 
-Cdp::Cdp(QWidget * parent) : QObject (parent)
+Cdp::Cdp(Taxipilot * parent)
 {
 
 	// get the debug-level
 	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 	QString opt = args->getOption("debug-level");
 	debug_level = opt.toInt();
-	opt_draw_platform_lines = args->isSet ("highlight_platforms");
+	draw_highlights = args->isSet ("highlight");
 	opt_thorough_check = args->isSet ("thorough_check");
 
 	// get the data-dirs
 	alt_image_prefix = image_prefix = KGlobal::dirs()->findResourceDir("images", "images_tag");
 
 	// store parent
-	parent_widget = parent;
+	tp = parent;
 
 	errors_occurred = false;
 
@@ -147,17 +153,16 @@ void Cdp::init_debug(QString in_file, bool exit_on_error)
 //    errors_occurred = false;
 	die_on_error = exit_on_error;
 	debug_in_file = in_file;
-	alt_data_prefix = *alt_path + "/";
-	alt_image_prefix = *alt_path + "/images/";
+	alt_data_prefix = alt_path + "/";
+	alt_image_prefix = alt_path + "/images/";
 }
 
 /** Returns inverse of errors_occurred, displaying a Dialog if errors_occured */
 bool Cdp::ok()
 {
 	if (errors_occurred) {
-		KMessageBox::error(parent_widget,
-						   i18n
-						   ("An error occured reading the configuration this mission.\nPlease check output to stderr for details."));
+		KMessageBox::sorry(tp->app,
+						   i18n ("An error occured reading the configuration for this mission.\n(See output to stderr for details)\nPlease select a different mission via Settings->Select Mission."));
 		errors_occurred = false;
 		return (false);
 	}
@@ -168,22 +173,31 @@ bool Cdp::ok()
 error and the program will abort in do_debug (). If *in_node is specified a backtrace is shown */
 void Cdp::debug_msg(QString message, int level, QDomNode const *in_node)
 {
+	QString color_code;
 
 	if (level <= debug_level) {	// generate output only if level <= debug_level
 		QString niveau;
 		QString bt_string = "";
 		switch (level) {
 		case 0:
+			color_code = "\033[01;31m";	// bright red
 			niveau = i18n("Fatal error");
 			break;
 		case 1:
+			color_code = "\033[01;35m";	// bright magenta
 			niveau = i18n("Serious warning");
 			break;
 		case 2:
+			color_code = "\033[01;32m";	// bright green
 			niveau = i18n("Warning");
 			break;
 		case 3:
-			niveau = i18n("Info");
+			color_code = "\033[0m";	// normal/black
+			niveau =  i18n("Info");
+			break;
+		case 4:
+			color_code = "\033[0m";	// normal/black
+			niveau =  i18n("Translations");
 			break;
 		}
 		if (in_node) {			// only create backtrace if in_node was supplied
@@ -194,12 +208,18 @@ void Cdp::debug_msg(QString message, int level, QDomNode const *in_node)
 				bt_string += i18n("#INVALID#") + "': ";
 			}
 		}
+		if (isatty (STDERR_FILENO)) {
+			fputs (color_code, stderr);		// ouput color_code only if this is a tty
+		}
 		fputs((i18n("In file '") + debug_in_file + "': " + bt_string + niveau + ": " + message + "\n"), stderr);
+		if (isatty (STDERR_FILENO)) {
+			fputs ("\033[0m", stderr);		// ouput color_code (back to normal) only if this is a tty
+		}
 	}
 
 	if (level == 0) {
 		if (die_on_error) {
-			KMessageBox::error(parent_widget, i18n("Error reading file '") + debug_in_file + "'\nExiting TaxiPilot.");
+			KMessageBox::error(tp, i18n("Error reading file '") + debug_in_file + "'\nExiting TaxiPilot.");
 			exit(1);
 		}
 		errors_occurred = true;
@@ -273,7 +293,7 @@ double Cdp::get_double_attribute(QString attribute, const QDomElement & element,
 
 /** Checks for the existence of a certain string XML-attribute in element. If given, returns its
 value. If not given or not existent, returns def and writes debugging message of level level */
-QString Cdp::get_string_attribute(QString attribute, const QDomElement & element, QString def, int level)
+QString Cdp::get_string_attribute(QString attribute, const QDomElement & element, QString def, int level, bool trans)
 {
 
 	if (!element.hasAttribute(attribute)) {
@@ -281,7 +301,17 @@ QString Cdp::get_string_attribute(QString attribute, const QDomElement & element
 		return def;
 	}
 
-	return element.attribute(attribute);
+	QString dummy = element.attribute(attribute);
+	if (!trans) {
+		return dummy;
+	}
+	if (translations.find (dummy) == translations.end ()) {
+		debug_msg(i18n("No translation found for string '") + dummy + "'", 4, &element);
+	} else {
+		debug_msg(i18n("Translating string '") + dummy + i18n ("' to '") + translations[dummy] + "'", 4, &element);
+		return (translations[dummy]);
+	}
+	return (dummy);
 }
 
 /** Checks for the existence of a certain bool XML-attribute in element. If given, returns its
@@ -324,6 +354,23 @@ QDomElement Cdp::open_config_file(QString const &file)
 		debug_msg(i18n("Error parsing XML-file"), 0);
 	}
 	f.close();
+		
+	// get translations-file (if any)
+	translations.clear ();
+	QFile t(file + "." + i18n("en_US"));	// TRANSLATORS: set "en_US" to your language
+	if (!t.open(IO_ReadOnly))
+		debug_msg(i18n("Could not find/open translations-file for reading"), 4);
+	else {
+		QTextStream l (&t);
+		while ( !l.eof() ) {        // until end of file...
+			QString dummy = l.readLine();
+			QString key = dummy.left (dummy.find (":::::"));
+			QString trans = dummy.right (dummy.length () - dummy.find (":::::") - 5);
+			translations.insert (key, trans);
+		}
+		t.close();
+	}
+	
 	return doc.documentElement();
 }
 
@@ -393,16 +440,27 @@ QDomElement Cdp::get_element(QString const &name, QDomElement const &parent, int
 
 	QDomNodeList list = get_node_list(name, parent, 0, 1, 1, level);
 
+#ifndef NEW_KDE
 	QDomNode node = list.item(0);
 	QDomElement e = node.toElement();
 
 	if (e.tagName() != name) {	// if no nodes by that name were found, you CANNOT reely on
-		// QT returning an empty list!
+		// QT (2) returning an empty list!
 		debug_msg(name + i18n("-element not given."), level, &parent);
 		QDomElement empty;
 		return (empty);
 	}
 	return (e);
+#else
+	if (!list.count ()) {
+		debug_msg(name + i18n("-element not given."), level, &parent);
+		QDomElement empty;
+		return (empty);
+	}
+
+	QDomNode node = list.item(0);
+	return (node.toElement ());
+#endif
 }
 
 /** Checks for the existence of a certain string XML-attribute in element. If given, checks
@@ -413,7 +471,7 @@ QString Cdp::get_multi_choice_attribute(QString attribute, const QDomElement & e
 
 	QString allowed_values = possible_values;
 
-	QString value = get_string_attribute(attribute, element, "", level);
+	QString value = get_string_attribute(attribute, element, "", level, false);	// do not translate
 
 	while (possible_values != "") {
 		int i = possible_values.findRev (';');
@@ -435,9 +493,9 @@ QString Cdp::get_multi_choice_attribute(QString attribute, const QDomElement & e
 	return def;
 }
 
-/** Returns true, if the commandline-option highlight_platforms was set, false otherwise */
-bool Cdp::draw_platform_lines (){
-	return (opt_draw_platform_lines);
+/** Returns true, if the commandline-option highlight was set, false otherwise */
+bool Cdp::draw_highlight (){
+	return (draw_highlights);
 }
 
 /** Returns true, if the commandline-option thorough_check was set, false otherwise */
@@ -487,7 +545,7 @@ void Cdp::setLastLoaded (QString &mission){
 /** Saves the current config (and emits a config_changed ()) */
 void Cdp::set_new_config (){
 
-	emit (config_changed ());
+	tp->config_changed ();
 
 // write settings
 	KConfig *config = kapp->config();
@@ -516,3 +574,13 @@ constructors to prevent crashes, while still making it possible to continue pars
 bool Cdp::isOk (){
 	return (!errors_occurred);
 }
+
+/** Checks, whether the given element is really of type/tagName "name" and returns false otherwise
++ outputs error-message at level level */
+bool Cdp::verify_tag (QString const &name, QDomElement const & element, int level) {
+	if (element.tagName () == name) {
+		return true;
+	}
+	debug_msg (i18n ("Tag of type '") + name + i18n ("' found, but '") + element.tagName () + i18n ("' expected."), level, &element);
+	return false;
+}
\ No newline at end of file
