@@ -2,7 +2,7 @@
                           taxi.cpp  -  description
                              -------------------
     begin                : Fri May 18 2001
-    copyright            : (C) 2001 by Thomas Friedrichsmeier
+    copyright            : (C) 2001, 2002 by Thomas Friedrichsmeier
     email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
  ***************************************************************************/
 
@@ -14,6 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
+#include "defines.h"
 
 #include <stdlib.h>
 
@@ -24,21 +25,15 @@
 #include "taxipilot.h"
 #include "cdp.h"
 #include "teleporter.h"
-
-#include "defines.h"
+#include "platform.h"
+#include "movingobject.h"
 
 Taxi::Taxi(const QString & file, QCanvas * canvas, int base_x, int base_y, Taxipilot * parent,
 		   bool for_credits_only):QCanvasSprite(0, canvas)
 {
 
 	cdp = parent->cdp;
-
-	if (cdp->for_real()) {
-		QObject::connect(parent, SIGNAL(gamePaused()), this, SLOT(pause()));
-		QObject::connect(parent, SIGNAL(gameResumed()), this, SLOT(resume()));
-		QObject::connect(parent, SIGNAL(taxiCrashed()), this, SLOT(crash()));
-		QObject::connect(cdp, SIGNAL (config_changed ()), this, SLOT (config_changed()));
-	}
+	tm = parent->tm;
 
 	idle_timer = anim_timer = flaps_anim_timer =-1;
 	lr = 0;
@@ -49,6 +44,7 @@ Taxi::Taxi(const QString & file, QCanvas * canvas, int base_x, int base_y, Taxip
 
 	flaps = new QCanvasSprite(0, canvas);
 	flaps->setZ(TAXI_FLAPS_Z);
+	flaps->setAnimated (false);
 
 	// prepare the flying-animations
 	flying_lr = new QCanvasSprite(0, canvas);
@@ -64,10 +60,10 @@ Taxi::Taxi(const QString & file, QCanvas * canvas, int base_x, int base_y, Taxip
 
 	// intialize flying-animation with a dummy (just to be sure)
 	if (cdp->for_real()) {
-		flying_lr->setFrame(flying_lr_frame = 0);
 		flying_lr->setSequence(flying_left.anim[lr]);
-		flying_ud->setFrame(flying_ud_frame = 0);
+		flying_lr->setFrame(flying_lr_frame = 0);
 		flying_ud->setSequence(flying_up.anim[lr]);
+		flying_ud->setFrame(flying_ud_frame = 0);
         }
 
 /* calculate offset x and y */
@@ -82,7 +78,7 @@ Taxi::Taxi(const QString & file, QCanvas * canvas, int base_x, int base_y, Taxip
 	}
 	off_y = double_off_y / 2;
 
-	paused = false;
+//	paused = false;
 	setZ(TAXI_Z);
 	dynamic_teleport = false;
 
@@ -112,7 +108,7 @@ Taxi::Taxi(const QString & file, QCanvas * canvas, int base_x, int base_y, Taxip
 Taxi::~Taxi()
 {
 
-	killTimers ();		// probably not necessary to do this explicitly, but who knows
+	tm->killTimers (this);		// probably not necessary to do this explicitly, but who knows
 	hide ();
 	flaps->hide ();
 	flying_lr->hide ();
@@ -148,8 +144,6 @@ a parameter instead of using some constant amount, so we may eventually
 abstract from keyboard input, and allow (analog) joystick input */
 void Taxi::up(double acc)
 {
-	double temp;
-
 	if (ready) {
 		if (fuel > 0) {
 
@@ -161,14 +155,16 @@ void Taxi::up(double acc)
 			flying_ud->setFrame(flying_ud_frame++);
 			flying_ud->show();
 
-			setYVelocity(temp = (yVelocity() + acc));
-			flaps->setYVelocity(temp);
-
 			if (landed) {
 				landed = false;
-				emit (take_off ());
+				platform_p->detach_taxi ();
+				setXVelocity ((platform_p->current_plat_vx () * (1000 / OBJECTS_AP) / cdp->fps ()));
+				setYVelocity ((platform_p->current_plat_vy () * (1000 / OBJECTS_AP) / cdp->fps ()));
+				wrap_moveBy (0, -2);		// To avoid getting "caught" again right away
 			}
 
+			setYVelocity(yVelocity() + acc);
+
 			fuel -= fuel_con;			
 		}
 	}
@@ -177,8 +173,6 @@ void Taxi::up(double acc)
 /** Signals we are going down. See up () for details. */
 void Taxi::down(double acc)
 {
-	double temp;
-
 	if (ready && !landed) {
 		if (fuel > 0) {
 
@@ -190,8 +184,7 @@ void Taxi::down(double acc)
 			flying_ud->setFrame(flying_ud_frame++);
 			flying_ud->show();
 
-			setYVelocity(temp = (yVelocity() + acc));
-			flaps->setYVelocity(temp);
+			setYVelocity(yVelocity() + acc);
 
 			fuel -= fuel_con;
 		}
@@ -201,13 +194,13 @@ void Taxi::down(double acc)
 /** See above */
 void Taxi::left(double acc)
 {
-	double temp;
-
 	if (ready && !(flaps_block && flaps_state) && !landed) {
 		if (fuel > 0) {
 
 			if (lr != 0) {
-				flaps->setSequence (cflaps_anim->anim[0]);
+				if (flaps_avail) {
+					flaps->setSequence (cflaps_anim->anim[0]);
+				}
 				setSequence (current_anim->anim[0]);
 				lr = 0;
 			}
@@ -220,8 +213,7 @@ void Taxi::left(double acc)
 			flying_lr->setFrame(flying_lr_frame++);
 			flying_lr->show();
 
-			setXVelocity(temp = (xVelocity() + acc));
-			flaps->setXVelocity(temp);
+			setXVelocity(xVelocity() + acc);
 
 			fuel -= fuel_con;
 		}
@@ -231,13 +223,13 @@ void Taxi::left(double acc)
 /** See above */
 void Taxi::right(double acc)
 {
-	double temp;
-
 	if (ready && !(flaps_block && flaps_state) && !landed) {
 		if (fuel > 0) {
 
 			if (lr != 1) {
-				flaps->setSequence (cflaps_anim->anim[1]);
+				if (flaps_avail) {
+					flaps->setSequence (cflaps_anim->anim[1]);
+				}
 				setSequence (current_anim->anim[1]);
 				lr = 1;
 			}
@@ -250,8 +242,7 @@ void Taxi::right(double acc)
 			flying_lr->setFrame(flying_lr_frame++);
 			flying_lr->show();
 
-			setXVelocity(temp = (xVelocity() + acc));
-			flaps->setXVelocity(temp);
+			setXVelocity(xVelocity() + acc);
 
 			fuel -= fuel_con;
 		}
@@ -277,8 +268,7 @@ void Taxi::read_taxi(const QString & file)
 	e = cdp->get_element("teleporting", xdocElement, 0);	// find the teleporting-animation
 	read_animation(&teleporting, e);
 
-	e = cdp->get_element("still", xdocElement, 0);	// find the standing-still-image
-	// not sure, we'll ever use different frames on this, though
+	e = cdp->get_element("still", xdocElement, 0);	// find the standing-still-image/animation
 	read_animation(&still, e);
 	still.continuous = true;
 
@@ -349,66 +339,6 @@ void Taxi::read_taxi(const QString & file)
 
 }
 
-/** Mostly everyting will be done inside: timing for all animations */
-void Taxi::timerEvent(QTimerEvent * e)
-{
-
-	if (!paused) {				// if paused, we won't do anything
-
-		if (e->timerId() == idle_timer) {	// this only sets off the actual animation
-			killTimer (idle_timer);
-			idle_timer = -1;
-			do_animation (&idle[(rand() % num_idle_animations)], &Taxi::go_idle);
-		} else if (e->timerId() == anim_timer) {
-			if (current_anim->direction > 0) {
-				if (ctaxi_frame < (current_anim->frames-1)) {
-					setFrame (++ctaxi_frame);
-				} else if (current_anim->continuous) {
-					setFrame (ctaxi_frame = 0);				
-				} else if (repetition < current_anim->repetitions) {
-					setFrame (ctaxi_frame = 0);
-					repetition++;
-				} else {
-					killTimer (anim_timer);
-					anim_timer = -1;
-					(this->*taxi_call_back) ();
-				}
-			} else if (current_anim->direction < 0) {
-				if (ctaxi_frame > 0) {
-					setFrame (--ctaxi_frame);
-				} else if (current_anim->continuous) {
-					setFrame (ctaxi_frame = (current_anim->frames -1));
-				} else if (repetition < current_anim->repetitions) {
-					setFrame (ctaxi_frame = (current_anim->frames -1));
-					repetition++;
-				} else {
-					killTimer (anim_timer);
-					anim_timer = -1;
-					(this->*taxi_call_back) ();
-				}
-			}
-		} else if (e->timerId() == flaps_anim_timer) {
-			if (cflaps_anim->direction > 0) {
-				if (cflaps_frame < (cflaps_anim->frames-1)) {
-					flaps->setFrame (++cflaps_frame);
-				} else {                             // flaps' animations are never continous or repetitous
-					killTimer (flaps_anim_timer);
-					flaps_anim_timer = -1;
-					(this->*flaps_call_back) ();
-				}
-			} else if (cflaps_anim->direction < 0) {
-				if (cflaps_frame > 0) {
-					flaps->setFrame (--cflaps_frame);
-				} else {
-					killTimer (flaps_anim_timer);
-					flaps_anim_timer = -1;
-					(this->*flaps_call_back) ();
-				}
-			}
-		}
-	}
-}
-
 /** Stops the currently running flying animation, if any (called from Taxipilot::do_chores ()) */
 void Taxi::stop_flying_animation()
 {
@@ -421,6 +351,7 @@ and speed. If appear_only is given, does not do disappearing animation. */
 void Taxi::teleport_to(int x, int y, double speedfactor_x, double speedfactor_y, bool appear_only, Teleporter *dynamic)
 {
 
+	if (landed) platform_p->detach_taxi ();
 	ready = landed = false;
 
 	if (!flaps_state) {			// make sure, flaps show up correctly after teleporting
@@ -447,10 +378,9 @@ void Taxi::teleport_to(int x, int y, double speedfactor_x, double speedfactor_y,
 	new_speed_x = xVelocity() * speedfactor_x;
 	new_speed_y = yVelocity() * speedfactor_y;
 	setVelocity(0, 0);
-	flaps->setVelocity(0, 0);
 
 	if (idle_timer != -1) {
-		killTimer(idle_timer);
+		tm->killTimer(idle_timer);
 		idle_timer = -1;
 	}
 
@@ -463,22 +393,6 @@ void Taxi::teleport_to(int x, int y, double speedfactor_x, double speedfactor_y,
 
 }
 
-/** Taxi is to stop everything it is doing */
-void Taxi::pause()
-{
-	paused = true;
-	setAnimated(false);
-	flaps->setAnimated(false);
-}
-
-/** Resume operation after pause () */
-void Taxi::resume()
-{
-	paused = false;
-	setAnimated(true);
-	flaps->setAnimated(true);
-}
-
 /** Returns the current fuel level in percent of the taxi (Used by Taxipilot to set the
 FuelDial) */
 int Taxi::fuel_state()
@@ -527,7 +441,10 @@ void Taxi::toggle_flaps()
 			if (flaps_state == 0) {	// flaps are in, going out
 				do_animation (&flaps_anim, &Taxi::flaps_out, true, true);
 			} else {			// flaps are out, going in
-				landed = false;
+				if (landed) {
+					platform_p->detach_taxi ();
+					landed = false;
+				}
 				do_animation (&flaps_anim, &Taxi::flaps_in, false, true);
 			}
 		}
@@ -563,7 +480,6 @@ void Taxi::reposition(int x, int y, double xv, double yv, bool align_bottom)
 	}
 
 	setVelocity(xv, yv);
-	flaps->setVelocity(xv, yv);
 }
 
 /** Tells the taxi it crashed. Does or will do stuff
@@ -572,9 +488,11 @@ sound and other things to do. */
 void Taxi::crash()
 {
 	setVelocity(0, 0);
-	flaps->setVelocity(0, 0);
 	do_animation (&flaps_anim, &Taxi::flaps_in, false, true);	
-	landed = false;
+	if (landed) {
+		landed = false;
+		platform_p->detach_taxi ();
+	}
 }
 
 /** Returns the (absolute) x-position the left landing-flaps is to be found at */
@@ -591,7 +509,7 @@ int Taxi::landing_flaps_x2()
 
 /** Returns the (absolute) y-position the landing-flaps are to be found at */
 int Taxi::landing_flaps_y(){
-	if (!flaps_avail) return (absY2());
+	if (!flaps_avail) return (BOTTOMEDGE());
 	return (flaps->boundingRect().bottom());
 }
 
@@ -627,11 +545,17 @@ void Taxi::read_animation(Anim * anim, QDomElement e)
 }
 
 /** A wrapper for moveBy, taking care of the flaps if appropriate */
-void Taxi::wrap_moveBy (int delta_x, int delta_y){
+void Taxi::wrap_moveBy (double delta_x, double delta_y){
 	moveBy (delta_x, delta_y);
 	flaps->moveBy (delta_x, delta_y);
 }
 
+/** Adjusts the taxi to a fractional position, so it doesn't wiggle on a moving platform */
+void Taxi::adjust (double fraction_x, double fraction_y) {
+	move (((int) x()) + fraction_x, ((int) y()) + fraction_y);
+	flaps->move (((int) x()) + fraction_x, ((int) y()) + fraction_y);
+}
+
 /** Sets of the animation anim for this taxi or it's flaps,  and triggers
 a function-call when completed. Reverse means: play in reverse order; flaps means, the animation is for the flaps */
 void Taxi::do_animation (Anim * anim, void (Taxi::*call_back) (), bool reverse, bool flaps) {
@@ -641,9 +565,9 @@ void Taxi::do_animation (Anim * anim, void (Taxi::*call_back) (), bool reverse,
 			cflaps_anim = anim;
 			flaps_call_back = call_back;
 			if (flaps_anim_timer != -1) {
-				killTimer (flaps_anim_timer);
+				tm->killTimer (flaps_anim_timer);
 			}	
-			flaps_anim_timer = startTimer (cflaps_anim->frame_period);
+			flaps_anim_timer = tm->startTimer (this, cflaps_anim->frame_period);
 			flaps_state = 1;
 			Taxi::flaps->show ();
 		}
@@ -652,9 +576,9 @@ void Taxi::do_animation (Anim * anim, void (Taxi::*call_back) (), bool reverse,
 		taxi_call_back = call_back;
 		repetition = 1;
 		if (anim_timer != -1) {
-			killTimer (anim_timer);
+			tm->killTimer (anim_timer);
 		}	
-		anim_timer = startTimer (current_anim->frame_period);
+		anim_timer = tm->startTimer (this, current_anim->frame_period);
 	}		
 
 	if (reverse) {
@@ -716,10 +640,10 @@ void Taxi::reappear (){
 /** Starts idle_timer (not idle_animation) */
 void Taxi::go_idle (){
 	if (idle_timer != -1) {
-		killTimer (idle_timer);		//just in case
+		tm->killTimer (idle_timer);		//just in case
 	}
 
-	idle_timer = startTimer ((rand() % 3000) + 2000);
+	idle_timer = tm->startTimer (this, (rand() % 3000) + 2000);
 
 	do_animation (&still, &Taxi::do_nothing);         // this is a continuous animation. no call_back needed
 }
@@ -732,10 +656,7 @@ void Taxi::do_nothing(){
 void Taxi::done_appearing (){
 
 	ready = true;	// we're ready to go
-
 	setVelocity(new_speed_x, new_speed_y);
-	flaps->setVelocity(new_speed_x, new_speed_y);
-
 	go_idle ();
 
 }
@@ -754,3 +675,76 @@ void Taxi::delete_animation (Anim &anim){
 void Taxi::config_changed(){
 	fuel_con = 20 / (double) cdp->cps ();
 }
+
+/** Mostly everyting will be done inside: timing for all animations */
+void Taxi::timerTick(int id)
+{
+
+	if (id == idle_timer) {	// this only sets off the actual animation
+		tm->killTimer (idle_timer);
+		idle_timer = -1;
+		do_animation (&idle[(rand() % num_idle_animations)], &Taxi::go_idle);
+	} else if (id == anim_timer) {
+		if (current_anim->direction > 0) {
+			if (ctaxi_frame < (current_anim->frames-1)) {
+				setFrame (++ctaxi_frame);
+			} else if (current_anim->continuous) {
+				setFrame (ctaxi_frame = 0);				
+			} else if (repetition < current_anim->repetitions) {
+				setFrame (ctaxi_frame = 0);
+				repetition++;
+			} else {
+				tm->killTimer (anim_timer);
+				anim_timer = -1;
+				(this->*taxi_call_back) ();
+			}
+		} else if (current_anim->direction < 0) {
+			if (ctaxi_frame > 0) {
+				setFrame (--ctaxi_frame);
+			} else if (current_anim->continuous) {
+				setFrame (ctaxi_frame = (current_anim->frames -1));
+			} else if (repetition < current_anim->repetitions) {
+				setFrame (ctaxi_frame = (current_anim->frames -1));
+				repetition++;
+			} else {
+				tm->killTimer (anim_timer);
+				anim_timer = -1;
+				(this->*taxi_call_back) ();
+			}
+		}
+	} else if (id == flaps_anim_timer) {
+		if (cflaps_anim->direction > 0) {
+			if (cflaps_frame < (cflaps_anim->frames-1)) {
+				flaps->setFrame (++cflaps_frame);
+			} else {                             // flaps' animations are never continous or repetitous
+				tm->killTimer (flaps_anim_timer);
+				flaps_anim_timer = -1;
+				(this->*flaps_call_back) ();
+			}
+		} else if (cflaps_anim->direction < 0) {
+			if (cflaps_frame > 0) {
+				flaps->setFrame (--cflaps_frame);
+			} else {
+				tm->killTimer (flaps_anim_timer);
+				flaps_anim_timer = -1;
+				(this->*flaps_call_back) ();
+			}
+		}
+	}
+}
+
+/** Reimplementation of the advance function to make sure that the flaps get moved
+accordingly. */
+void Taxi::advance (int phase){
+	if (phase == 1) {
+		moveBy (xVelocity (), yVelocity ());
+		flaps->moveBy (xVelocity (), yVelocity ());
+	}
+}
+
+/** Tell the taxi it has landed on the specified paltform */
+void Taxi::land (Platform *on) {
+	landed = true;
+	platform_p = on;
+	platform_p->attach_taxi (this);
+}
\ No newline at end of file
