@@ -2,7 +2,7 @@
                           taxipilot.cpp  -  description
                              -------------------
     begin                : Don Mai 17 20:03:43 CEST 2001
-    copyright            : (C) 2001 by Thomas Friedrichsmeier
+    copyright            : (C) 2001, 2002 by Thomas Friedrichsmeier
     email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
  ***************************************************************************/
 
@@ -14,11 +14,12 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-
+#include "defines.h"
 
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kfiledialog.h>
+#include KAPP
 
 #include <qlayout.h>
 #include <qdom.h>
@@ -44,8 +45,8 @@
 #include "centerofgravity.h"
 #include "platform.h"
 #include "message.h"
-
-#include "defines.h"
+#include "timemanager.h"
+#include "hotarea.h"
 
 Taxipilot::Taxipilot(TaxipilotApp * parent, const char *name):QWidget(parent, name)
 {
@@ -57,8 +58,7 @@ Taxipilot::Taxipilot(TaxipilotApp * parent, const char *name):QWidget(parent, na
 	start_screen_active = false;
 	passengers_created = false;
 
-	setFocusPolicy(QWidget::StrongFocus);	// get Focus
-	setFocus();
+	base_timer = passenger_timer = scroll_timer = credits_timer = canvas_timer = -1;
 
 	srand((unsigned int) time((time_t *) NULL));	// randomize
 
@@ -171,12 +171,14 @@ Taxipilot::Taxipilot(TaxipilotApp * parent, const char *name):QWidget(parent, na
 
 	cdp = new Cdp(this);
 
+	tm = new TimeManager (50, 10, cdp);
+	init_timer = tm->startTimer (this, 10);		// this timer will get active after a.exec () in main and then fire initialization.
+	pause_tm = new TimeManager (10, 10, cdp);
+	pause_tm->pause ();
 	message_handler = new Message (this, view); // needs to know about cdp
 
 	passenger_exists = false;
 	config_changed ();		// this updates what we store about the config
-	connect (cdp, SIGNAL (config_changed ()), SLOT (config_changed ()));	// and this keeps us up to date
-
 }
 
 Taxipilot::~Taxipilot()
@@ -192,7 +194,6 @@ Taxipilot::~Taxipilot()
 	delete[]passenger_files;
 	delete fader;
 	delete message_handler;
-	delete_credits();
 
 	if (start_screen_active) {
 		delete_start_screen ();
@@ -200,12 +201,15 @@ Taxipilot::~Taxipilot()
 
 	if (game_running) {
 		delete taxi;
+		delete level;
 	}
 
+	delete tm;
+	delete pause_tm;
+	delete cdp;
+
 	delete view;
 	delete canvas;
-	
-	delete cdp;
 }
 
 /** This function really would be part of the constructor. It initializes Taxipilot by reading
@@ -216,16 +220,16 @@ in turn depends on having an instance of Taxipilot.
 If you can think of something more elegant, please don't hesitate to tell me. */
 void Taxipilot::init()
 {
-        num_passengers_on_start_screen = num_passenger_files = num_level_files = 0;
-	
+	num_passengers_on_start_screen = num_passenger_files = num_level_files = 0;
+
+	setFocusPolicy(QWidget::StrongFocus);	// get Focus
+	setFocus();
+
+	cdp->init_debug (i18n ("No file opened yet"), false);	
 	readMission(cdp->prefix_check(cdp->startupMission (), false));
 
-	base_timer = -1;			// just in case. No idea whether this is neccessary.
-	passenger_timer = -1;
-	scroll_timer = -1;
-	credits_timer = -1;
-
 	game_running = false;
+	std_resumable = true;
 
 	if (cdp->ok()) {
 		display_start_screen();
@@ -243,13 +247,13 @@ bool Taxipilot::onActivePlatform()
 
 
 /* This function is to handle virtually everything of significance */
-void Taxipilot::timerEvent(QTimerEvent * e)
-{								// loads to be done yet
+void Taxipilot::timerTick(int id)
+{						
 
-	if ((!game_paused) && game_running) {	// do nothing if paused.
-		if (e->timerId() == scroll_timer) {
+	if (game_running) {	// do nothing if paused.
+		if (id == scroll_timer) {
 			do_scroll();
-		} else if (e->timerId() == base_timer) {	// base timer
+		} else if (id == base_timer) {	// base timer
 
 			// whatever may happen, do the chores ( gravity only if not landed )
 			do_chores();
@@ -282,6 +286,12 @@ void Taxipilot::timerEvent(QTimerEvent * e)
 				}			
 			}
 
+			// check for hot-areas here
+			QRect dummy = taxi->boundingRect ();			
+			for (int i=0; i < level->num_areas; i++) {
+				level->areas[i]->check (dummy);
+			}
+
 			if (taxi->ready) {	// means: not teleporting already
 
 				for (int i = 0; i < level->num_teleporters; i++) {
@@ -375,7 +385,7 @@ void Taxipilot::timerEvent(QTimerEvent * e)
 				}
 			}
 
-		} else if (e->timerId() == passenger_timer) {	// create new passenger
+		} else if (id == passenger_timer) {	// create new passenger
 
 			int rand_num;
 			do {
@@ -385,19 +395,21 @@ void Taxipilot::timerEvent(QTimerEvent * e)
 
 			passenger = all_passengers[(rand() % num_passenger_files)];
 
-			passenger->initialize (level->platform[active_platform]->current_plat_base_x (), level->platform[active_platform]->current_plat_y ());
-			level->platform[active_platform]->attach_passenger (passenger);
+			passenger->initialize (level->platform[active_platform]->current_plat_base_x (), level->platform[active_platform]->current_plat_y (), level->platform[active_platform]);
 			passenger_exists = true;
 
-			killTimer(passenger_timer);
+			tm->killTimer(passenger_timer);
 			passenger_timer = -1;
 
 			// everyting else is taken care of by passenger_ready ()
-		}
 
+		} else if (id == canvas_timer) {	// advance taxi and update canvas
+			taxi->advance (1);
+			canvas->update ();
+		}
 	} else {					// stuff to do only when game is paused
 
-		 if (e->timerId() == credits_timer) {
+		 if (id == credits_timer) {
 			Credit_item *dummy;
 			int store_current = credits.at();
 			if (!credits.current()->item->visible()) {	// set off a new item
@@ -429,7 +441,15 @@ void Taxipilot::timerEvent(QTimerEvent * e)
 			} else {
 				credits.at(credits.count() - 1);
 			}
-		}						// else yet another timer
+
+		} else if (id == canvas_timer) {	// advance credits and update canvas
+			canvas->advance ();
+		} else if (id == init_timer) {
+			tm->killTimer (init_timer);
+			init_timer = -1;
+			init ();
+		}
+
 	}
 }
 
@@ -534,7 +554,7 @@ void Taxipilot::delivery_complete()
 	passenger->discard ();
 	passenger_exists = false;
 	if (level->deliveries_to_go) {
-		passenger_timer = startTimer(PASSENGER_TP);	// to be randomized
+		passenger_timer = tm->startTimer(this, PASSENGER_TP);	// to be randomized
 	} else {					// this level is complete!
 
 		int message_id = NEXT_LEVEL;
@@ -561,7 +581,7 @@ void Taxipilot::passenger_ready()
 
 	passenger_waiting = true;
 	if (onActivePlatform()) {	// in case we are already there
-		emit(start_boarding(taxi->absX() + (int) (taxi->width() / 2)));
+		passenger->pick_up (taxi->LEFTEDGE() + (int) (taxi->width() / 2));
 	}
 }
 
@@ -573,8 +593,8 @@ void Taxipilot::do_gravity()
 		double xv = taxi->xVelocity() + level->horiz_gravity;	// do the basic gravity right in place;
 		double yv = taxi->yVelocity() + level->base_gravity;
 
-		int taxi_pos_x = taxi->absX() + (taxi->width() / 2);
-		int taxi_pos_y = taxi->absY() + (taxi->height() / 2);
+		int taxi_pos_x = taxi->LEFTEDGE() + (taxi->width() / 2);
+		int taxi_pos_y = taxi->TOPEDGE() + (taxi->height() / 2);
 
 		for (int i = 0; i < level->num_grav_centers; i++) {	// do the advanced gravity
 			level->grav_center[i]->calculate_gravity (taxi_pos_x, taxi_pos_y, &xv, &yv);
@@ -606,7 +626,7 @@ void Taxipilot::landing_func()
 	// first of all: stop
 	speedDial->setValue(taxi->speed = 0);
 	taxi->reposition(-1, level->platform[on_platform]->current_plat_y (), 0, 0, true);
-	taxi->landed = true;
+	taxi->land (level->platform[on_platform]);
 
 	// now let's see, if there's any fuel here
 	if (level->platform[on_platform]->fuel_step > 0) {
@@ -623,11 +643,10 @@ void Taxipilot::landing_func()
 
 		if (passenger_waiting) {
 
-			emit(start_boarding(taxi->absX() + (int) (taxi->width() / 2)));	// you may now come aboard!
+			passenger->pick_up (taxi->LEFTEDGE() + (int) (taxi->width() / 2));	// you may now come aboard!
 		} else if (passenger_onboard) {
 
-			emit(delivered_passenger((taxi->absX() + (int) (taxi->width() / 2)), level->platform[active_platform]->current_plat_y (), level->platform[active_platform]->current_plat_base_x ()));	// there you go!
-			level->platform[active_platform]->attach_passenger (passenger);
+			passenger->got_delivered ((taxi->LEFTEDGE() + (int) (taxi->width() / 2)), level->platform[active_platform]); 	// there you go!
 			passenger_onboard = false;
 			passenger_waiting = false;
 
@@ -653,10 +672,9 @@ void Taxipilot::crash(int how)
 
 	if (--ships < 0) {
 		message_id += GAME_OVER;
-		emit (gameOver());
 	}
 
-	emit(taxiCrashed());
+	taxi->crash ();
 
 	display_message(message_id + how);
 
@@ -677,6 +695,7 @@ void Taxipilot::newGame()
 		for (int i=0; i < num_passenger_files; i++) {
 			all_passengers[i] = new Passenger(passenger_files[i], canvas, this);
 		}
+		passengers_created = true;
 	}
 
 	level = new Level(level_files[(current_level = 0)], canvas, this);
@@ -697,15 +716,15 @@ void Taxipilot::newGame()
 
 // initialize timers and variables
 
-	base_timer = startTimer(1000 /  cdp->cps ());
-	scroll_timer = startTimer(SCROLL_TP);
+	base_timer = tm->startTimer(this, 1000 /  cdp->cps ());
+	scroll_timer = tm->startTimer(this, 1000 / cdp->fps ());
 
 	passenger_waiting = false;
 	passenger_onboard = false;
 	passenger_exists = false;
 	fueling = false;
 
-	passenger_timer = startTimer(PASSENGER_TP);	// randomize ?
+	passenger_timer = tm->startTimer(this, PASSENGER_TP);	// randomize ?
 
 	game_running = true;
 
@@ -718,39 +737,38 @@ void Taxipilot::quitGame()
 {
 	pauseGame();
 	if (3 == KMessageBox::questionYesNo(this, i18n("Really wanna quit?"), "", i18n("'course"), i18n("Never!"))) {
-		exit(0);
+		kapp->quit ();
 	}
 	resumeGame();
 }
 
-/** Pauses the game. This might mess up the timers and stuff.
-Also any class having timers will have to have an eye on this. */
+/** Pauses the game. */
 void Taxipilot::pauseGame()
 {
 	if (game_running) {
-
-		emit(gamePaused());
-		game_paused = true;
 		app->gamePause->setEnabled(false);
-		app->gameResume->setEnabled(true);
+                if (std_resumable) {
+			display_message (PAUSE);
+		}
 		lastStatus = statusLabel->text();
 		statusLabel->setText(i18n("Game paused"));
 		fader->show();
 		canvas->update();
-
 	}
+	tm->pause ();
+	pause_tm->resume();
 }
 
 /** Resumes a paused game */
 void Taxipilot::resumeGame()
 {
-	emit(gameResumed());
-	game_paused = false;
 	app->gamePause->setEnabled(true);
-	app->gameResume->setEnabled(false);
-	canvas->setAdvancePeriod(1000 / cdp->fps ());
+	canvas_timer = tm->setLastTimer (this, 1000 / cdp->fps ());
 	statusLabel->setText(lastStatus);
 	fader->hide();
+	tm->resume ();
+	pause_tm->pause();
+	std_resumable = true;
 }
 
 /** Selects a new Mission (i.e. a list of level- and
@@ -772,9 +790,9 @@ void Taxipilot::selectMission()
 
 		if (cdp->ok()) {
 			display_start_screen();
-			canvas->setAdvancePeriod(0);
+			pauseGame ();
 			if (cdp->ok()) {
-				canvas->setAdvancePeriod(1000 / cdp->fps ());
+				resumeGame();
 				cdp->setLastLoaded (file);
 				app->gameNew->setEnabled(true);
 			} else {
@@ -805,7 +823,7 @@ void Taxipilot::readMission(const QString & file)
 		return;					// Errors at this point ARE fatal!
 
 	startscreen_file =
-		cdp->prefix_check(cdp->get_string_attribute("startscreen", xdocElement, "default_start_screen.png", 1), true);
+		cdp->prefix_check(cdp->get_string_attribute("startscreen", xdocElement, "default_start_screen.jpg", 1), true);
 
 	e = cdp->get_element("initials", xdocElement, 1);	// find the initials-section
 	shipsLCD->display((ships = initial_ships = cdp->get_int_attribute("ships", e, 1, 99, 3, 1)));
@@ -888,9 +906,8 @@ void Taxipilot::ask_abortGame()
 {
 
 	pauseGame();
-	canvas->setAdvancePeriod(0);
 	if (3 != KMessageBox::questionYesNo(this, i18n("This will quit the current game!\nAre you sure?"))) {
-		resumeGame();
+//		resumeGame();
 		return;
 	}
 	abortGame();
@@ -903,8 +920,9 @@ void Taxipilot::abortGame()
 
 	pauseGame();				// Usually we've already done this in ask_abortGame. It should'nt harm, though
 
+	message_handler->kill ();
 	game_running = false;
-	killTimers();
+	tm->killTimers(this);
 
 	if (passenger_exists) {
 		passenger->discard();
@@ -914,20 +932,21 @@ void Taxipilot::abortGame()
 	delete taxi;
 
 	app->gamePause->setEnabled(false);
-	app->gameResume->setEnabled(false);
 	app->gameAbort->setEnabled(false);
 	app->gameNew->setEnabled(true);
 
-	canvas->setAdvancePeriod(0);
+//	tm->killLastTimer ();
+//	canvas->setAdvancePeriod(0);
 
 // in case there is still a message 
 	fader->hide();
 	canvas->update();
 
-	base_timer = -1;			// just in case. No idea whether this is neccessary.
+	base_timer = -1;
 	passenger_timer = -1;
 	scroll_timer = -1;
 	credits_timer = -1;
+	canvas_timer = -1;
 
 	statusLabel->setText(i18n("No Game running"));
 
@@ -952,16 +971,14 @@ void Taxipilot::display_start_screen()
 
 	for (int i = 0; i < num_passengers_on_start_screen; i++) {
 		passengers_on_start_screen[i].pointer =	new Passenger(passenger_files[passengers_on_start_screen[i].id], canvas, this);
-		passengers_on_start_screen[i].pointer->initialize (passengers_on_start_screen[i].x, passengers_on_start_screen[i].y, true);
+		passengers_on_start_screen[i].pointer->initialize (passengers_on_start_screen[i].x, passengers_on_start_screen[i].y, 0);
 	}
 
 	if (taxi_on_start_screen) {
 		taxi = new Taxi(taxi_file, canvas, taxi_start_screen_x, taxi_start_screen_y, this);
 	}
 
-/* For some strange reason we have to used AdvancePeriod here. UpdatePeriod does (contrary to the
-documentation) not appear to be overwriteable by AdvancePeriod. */
-	canvas->setAdvancePeriod(1000 / cdp->fps ());
+	resumeGame ();		// restart timers
 
 	app->setCaption(mission_title);
 
@@ -994,12 +1011,16 @@ void Taxipilot::delete_start_screen()
 void Taxipilot::display_message(int message_id)
 {
 
-	pauseGame();
+	std_resumable = false;
+	QString message = "";
+
+	if (message_id != PAUSE) pauseGame();
+	else {
+		message = i18n ("Game paused\nto continue:");
+	}
 
 	event_after_message = message_id;
 
-	QString message = "";
-
 	if (message_id & CRASH) {
 		if (message_id & CRASH_PASSENGER) {
 			message = i18n("You just killed an innocent passenger!\nAre you mad?!?");
@@ -1033,8 +1054,7 @@ void Taxipilot::display_message(int message_id)
 		event_after_message |= LOW_FUEL;
 	}
 
-	QString dummy = SLOT (do_event_after_message ());
-	message_handler->display_message (message, app->ack_key_name, dummy);
+	message_handler->display_message (message, app->ack_key_name);
 
 }
 
@@ -1066,7 +1086,7 @@ void Taxipilot::do_event_after_message()
 			passenger_onboard = false;
 			passenger_waiting = false;
 			destinationLabel->setText(i18n("Nowhere"));	// update display
-			statusLabel->setText(i18n("Idle"));
+			lastStatus = i18n("Idle");
 
 			if (!level->deliveries_to_go) {	// If this was the last passenger to deliver, we would get stuck.
 				level->deliveries_to_go = 1;
@@ -1077,6 +1097,7 @@ void Taxipilot::do_event_after_message()
 
 	} else if (event_after_message & NEXT_LEVEL) {	// next level
 
+		taxi->landed = false;	// otherwise taxi would try to detach itself from (to be deleted) platform
 		delete level;
 		level = new Level(level_files[current_level], canvas, this);
 		view->setFixedSize(level->visible_width + 3, level->visible_height + 3);	// this has to happend after initialization of level, for this is where the canvas is initialized!
@@ -1094,7 +1115,7 @@ void Taxipilot::do_event_after_message()
 		passenger_exists = false;
 		fueling = false;
 
-		passenger_timer = startTimer(PASSENGER_TP);	// next passenger may appear, too
+		passenger_timer = tm->startTimer(this, PASSENGER_TP);	// next passenger may appear, too
 	}							// else if MISSION_COMPLETE
 
 	resumeGame();				// this will set the AdvancePeriod.
@@ -1110,8 +1131,7 @@ void Taxipilot::do_scroll()
 	if ((taxi->x() - current_offset_x) < level->margin) {
 		if (current_offset_x > 0) {
 			dummy =
-				(int) ((level->margin - (taxi->x() - current_offset_x)) +
-					   (taxi->xVelocity() * (ADVANCE_PERIOD / SCROLL_TP)));
+				(int) ((level->margin - (taxi->x() - current_offset_x)) + taxi->xVelocity());
 			if (current_offset_x > dummy) {
 				current_offset_x -= dummy;
 			} else {
@@ -1119,10 +1139,10 @@ void Taxipilot::do_scroll()
 			}
 			changes = true;
 		}
-	} else if ((taxi->absX2() - current_offset_x) > (level->visible_width - level->margin)) {
+	} else if ((taxi->RIGHTEDGE() - current_offset_x) > (level->visible_width - level->margin)) {
 		dummy =
-			((taxi->absX2() - current_offset_x) - (level->visible_width - level->margin)) +
-			(int) (taxi->xVelocity() * (ADVANCE_PERIOD / SCROLL_TP));
+			((taxi->RIGHTEDGE() - current_offset_x) - (level->visible_width - level->margin)) +
+			(int) (taxi->xVelocity());
 		if (current_offset_x < level->max_offset_x) {
 			if ((level->max_offset_x - current_offset_x) > dummy) {
 				current_offset_x += dummy;
@@ -1138,7 +1158,7 @@ void Taxipilot::do_scroll()
 		if (current_offset_y > 0) {
 			dummy =
 				(int) ((level->margin - (taxi->y() - current_offset_y)) +
-					   (taxi->yVelocity() * (ADVANCE_PERIOD / SCROLL_TP)));
+					   (taxi->yVelocity()));
 			if (current_offset_y > dummy) {
 				current_offset_y -= dummy;
 			} else {
@@ -1146,11 +1166,11 @@ void Taxipilot::do_scroll()
 			}
 			changes = true;
 		}
-	} else if ((taxi->absY2() - current_offset_y) > (level->visible_height - level->margin)) {
+	} else if ((taxi->BOTTOMEDGE() - current_offset_y) > (level->visible_height - level->margin)) {
 		if (current_offset_y < level->max_offset_y) {
 			dummy =
-				((taxi->absY2() - current_offset_y) - (level->visible_height - level->margin)) +
-				(int) (taxi->yVelocity() * (ADVANCE_PERIOD / SCROLL_TP));
+				((taxi->BOTTOMEDGE() - current_offset_y) - (level->visible_height - level->margin)) +
+				(int) (taxi->yVelocity());
 			if ((level->max_offset_y - current_offset_y) > dummy) {
 				current_offset_y += dummy;
 			} else {
@@ -1245,7 +1265,7 @@ void Taxipilot::generate_credits()
 	for (i = 0; i < num_passenger_files; i++) {
 
 		Passenger *credits_passenger = new Passenger(passenger_files[i], canvas, this);
-		credits_passenger->initialize (0, 0, true, true);
+		credits_passenger->initialize (0, 0, 0, true);
 
 		dummy = new Credit_item;
 		text_dummy = new QCanvasText("'", credits_font, canvas);
@@ -1322,7 +1342,7 @@ void Taxipilot::generate_credits()
 	credits.first();			// go to first item
 	first_visible_credit = 0;
 	last_visible_credit = -1;
-	credits_timer = startTimer(CREDITS_TP);	// set off animation
+	credits_timer = tm->startTimer(this, CREDITS_TP);	// set off animation
 
 }
 
@@ -1338,7 +1358,7 @@ void Taxipilot::delete_credits()
 		credits.remove();		// since we called first (), this removes the first item.
 	}
 
-	killTimer(credits_timer);
+	tm->killTimer(credits_timer);
 	credits_timer = -1;
 }
 
@@ -1358,7 +1378,7 @@ void Taxipilot::append_credit(Credit_item * item, int space_after)
 
 /** Pause the game when focus gets lost. */
 void Taxipilot::focusOutEvent (QFocusEvent *e){
-	app->gamePause->activate();
+	if (game_running) app->gamePause->activate();
 }
 
 /** Receives a signal that the config changed. */
@@ -1369,10 +1389,17 @@ void Taxipilot::config_changed(){
 		tip_step = 0.2 / (double) cdp->cps () * passenger->impatience;
 	}
 	if (game_running) {
-		level->config_changed ();	// the level is no QObject, so we just tell it by hand.
+		level->config_changed ();	// the level is no QObject, so we just tell it by hand. (The level will tell the centers of gravity)
+		taxi->config_changed ();
 	}
-	killTimer (base_timer);
-	base_timer = startTimer (1000 / cdp->cps ());		// AdvancePeriod is set in resumeGame ()
+	if (base_timer != -1) {
+		tm->killTimer (base_timer);
+	}
+	base_timer = tm->startTimer (this, 1000 / cdp->cps ());		// AdvancePeriod is set in resumeGame ()
+	if (scroll_timer != -1) {
+		tm->killTimer (scroll_timer);
+	}
+	scroll_timer = tm->startTimer (this, 1000 / cdp->fps ());
 
 	speedDial->setMaxValue (10000 / cdp->fps ());
 	speedDial->setLimits (-1, 4000 / cdp->fps ());
